@@ -45,10 +45,11 @@ const (
 	etcdClientCAConfigMap         = "openshift-config/etcd-metrics-serving-ca"
 	telemeterCABundleConfigMap    = "openshift-monitoring/telemeter-trusted-ca-bundle"
 	alertmanagerCABundleConfigMap = "openshift-monitoring/alertmanager-trusted-ca-bundle"
+	grpcTLS                       = "openshift-monitoring/grpc-tls"
 )
 
 type Operator struct {
-	namespace string
+	namespace, namespaceUserWorkload string
 
 	configMapName string
 	images        map[string]string
@@ -67,18 +68,19 @@ type Operator struct {
 	reconcileErrors   prometheus.Counter
 }
 
-func New(config *rest.Config, version, namespace, namespaceSelector, configMapName string, images map[string]string) (*Operator, error) {
+func New(config *rest.Config, version, namespace, namespaceUserWorkload, namespaceSelector, configMapName string, images map[string]string) (*Operator, error) {
 	c, err := client.New(config, version, namespace, namespaceSelector)
 	if err != nil {
 		return nil, err
 	}
 
 	o := &Operator{
-		images:        images,
-		configMapName: configMapName,
-		namespace:     namespace,
-		client:        c,
-		queue:         workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "cluster-monitoring"),
+		images:                images,
+		configMapName:         configMapName,
+		namespace:             namespace,
+		namespaceUserWorkload: namespaceUserWorkload,
+		client:                c,
+		queue:                 workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "cluster-monitoring"),
 	}
 
 	o.secretInf = cache.NewSharedIndexInformer(
@@ -170,6 +172,7 @@ func (o *Operator) Run(stopc <-chan struct{}) error {
 	}
 
 	go o.cmapInf.Run(stopc)
+	go o.secretInf.Run(stopc)
 	go o.kubeSystemCmapInf.Run(stopc)
 
 	klog.V(4).Info("Waiting for initial cache sync.")
@@ -234,6 +237,7 @@ func (o *Operator) handleEvent(obj interface{}) {
 	case etcdClientCAConfigMap:
 	case telemeterCABundleConfigMap:
 	case alertmanagerCABundleConfigMap:
+	case grpcTLS:
 	default:
 		klog.V(5).Infof("ConfigMap or Secret (%s) not triggering an update.", key)
 		return
@@ -291,15 +295,17 @@ func (o *Operator) sync(key string) error {
 	config := o.Config(key)
 	config.SetImages(o.images)
 
-	factory := manifests.NewFactory(o.namespace, config)
+	factory := manifests.NewFactory(o.namespace, o.namespaceUserWorkload, config)
 
 	tl := tasks.NewTaskRunner(
 		o.client,
 		[]*tasks.TaskSpec{
 			tasks.NewTaskSpec("Updating Prometheus Operator", tasks.NewPrometheusOperatorTask(o.client, factory)),
+			tasks.NewTaskSpec("Updating user workload Prometheus Operator", tasks.NewPrometheusOperatorUserWorkloadTask(o.client, factory, config.UserWorkloadConfig)),
 			tasks.NewTaskSpec("Updating Cluster Monitoring Operator", tasks.NewClusterMonitoringOperatorTask(o.client, factory)),
 			tasks.NewTaskSpec("Updating Grafana", tasks.NewGrafanaTask(o.client, factory)),
 			tasks.NewTaskSpec("Updating Prometheus-k8s", tasks.NewPrometheusTask(o.client, factory, config)),
+			tasks.NewTaskSpec("Updating Prometheus-user-workload", tasks.NewPrometheusUserWorkloadTask(o.client, factory, config.UserWorkloadConfig)),
 			tasks.NewTaskSpec("Updating Alertmanager", tasks.NewAlertmanagerTask(o.client, factory)),
 			tasks.NewTaskSpec("Updating node-exporter", tasks.NewNodeExporterTask(o.client, factory)),
 			tasks.NewTaskSpec("Updating kube-state-metrics", tasks.NewKubeStateMetricsTask(o.client, factory)),
@@ -307,6 +313,7 @@ func (o *Operator) sync(key string) error {
 			tasks.NewTaskSpec("Updating prometheus-adapter", tasks.NewPrometheusAdapterTaks(o.namespace, o.client, factory)),
 			tasks.NewTaskSpec("Updating Telemeter client", tasks.NewTelemeterClientTask(o.client, factory, config.TelemeterClientConfig)),
 			tasks.NewTaskSpec("Updating configuration sharing", tasks.NewConfigSharingTask(o.client, factory)),
+			tasks.NewTaskSpec("Updating Thanos Querier", tasks.NewThanosQuerierTask(o.client, factory, config.UserWorkloadConfig)),
 		},
 	)
 
